import { Component, OnDestroy, OnInit } from '@angular/core';
import { CookieService } from 'ngx-cookie-service';
import { Subscription } from 'rxjs';
import { environment } from '../../../../environments/environment';
import { notificationMessages } from '../../../../i18n/traductions';
import { Notification } from '../../../core/models';
import { NotificationService } from '../../../core/services';
import { Metadata } from '../../../shared/models';
import { UserService } from '../../../user/services';
import { Resource } from '../../models';
import { DatasetDetailService, ResourcesService } from '../../services';

@Component({
  selector: 'app-dataset-api',
  templateUrl: './dataset-api.component.html',
  styleUrls: ['./dataset-api.component.scss'],
})
export class DatasetAPIComponent implements OnInit, OnDestroy {

  metadata: Metadata;
  resources: Resource[];
  displayLicenseModal = false;
  sub: Subscription;
  hasBeenInitialized = false;

  cguModalIsOpened: boolean;

  isSample = false;

  // To display in the template
  queryableResources: Resource[];

  constructor(
    private _datasetDetailService: DatasetDetailService,
    private _resourcesService: ResourcesService,
    private _notificationService: NotificationService,
    private _cookieService: CookieService,
    private _userService: UserService,
  ) { }

  ngOnInit() {
    this.resources = [];
    this.queryableResources = [];
    this.isSample = this._datasetDetailService.dataset.editorialMetadata.isSample;
    this.cguModalIsOpened = (this._cookieService.get(environment.cguAcceptedCookieName) !== 'true') &&
      !this._userService.userIsSignedIn;

    this.initialize();
    this._resourcesService.getResources().subscribe(
      (results) => {
        this.resources = results;

        this.initialize();
        this.sub = this._datasetDetailService.dataset$.subscribe(() => {
          this.initialize();
        });
      },
      (err) => {
        this._notificationService.notify(new Notification({
          type: 'error',
          message: `${notificationMessages.resources.initializationError}`,
        }));
      },
    );

  }

  ngOnDestroy() {
    if (this.sub) {
      this.sub.unsubscribe();
    }
  }

  // Resources are organized on 2 levels :
  // -> 1st level: by metadata link description
  // -> 2nd level: for each link description in 3 categories:
  //    - queryable
  //    - downloadable
  //    - other
  // How to organize: we compare the type of one resource from our resource service
  // and the ES metadata link property. If there is a match, the service will let us know if the resource
  // is queryable or downloadable. If there is no match and the ES resource cannot be found in the service,
  // we put it in "others" resources catagory.
  initialize() {
    this.metadata = this._datasetDetailService.datasetMetadata;
    if (this.metadata && this.resources.length > 0 && !this.hasBeenInitialized) {
      this.hasBeenInitialized = true;
      if (this.metadata.link) {
        this.metadata.link.forEach((link) => {
          if (link.service) {
            this.resources.forEach((resource) => {
              const resourceCopy = Object.assign({}, resource);
              resourceCopy.metadataLink = link;
              if (link.service === resource.type && resourceCopy.isQueryable) {
                if (!this.queryableResources[link.description]) {
                  this.queryableResources[link.description] = [];
                }
                this.queryableResources[link.description].push(resourceCopy);
              }
            });
          }
        });
      }
    }
  }

  toggleLicenseModal() {
    this.displayLicenseModal = !this.displayLicenseModal;
  }

  get hasQueryableResources() {
    return Object.keys(this.queryableResources).length > 0;
  }

  cguAccepted() {
    this.cguModalIsOpened = false;
  }
}
