import { Component, Input, OnInit } from '@angular/core';
import { DomSanitizer, SafeHtml } from '@angular/platform-browser';
import { communeInsee } from '../../../../../../assets/resources/commune-insee';
import { geosource } from '../../../../../../i18n/traductions';
import { NotificationService } from '../../../../../core/services';
import { linkFormats, Metadata } from '../../../../../shared/models';
import { Format, Projection, Resource } from '../../../../models';
import { DatasetDetailService } from '../../../../services';

@Component({
  selector: 'app-resource-queryable',
  templateUrl: './resource-queryable.component.html',
  styleUrls: ['./resource-queryable.component.scss'],
})
export class ResourceQueryableComponent implements OnInit {
  linkFormats = linkFormats;

  @Input() resource: Resource;
  @Input() metadata: Metadata;
  @Input() projections: Projection[];

  downloadMessage = geosource.downloads.download;

  messageClipboard: string;
  messageClipboardRoot: string;

  queryableUrl: string;
  queryableUrlToDisplay: SafeHtml;
  allSettingsOK: boolean = true;

  shpProjections = {
    'EPSG:4171': 'EPSG:4171',
    'EPSG:3946': 'EPSG:3946',
    'EPSG:2154': 'EPSG:2154',
    'EPSG:4326': 'EPSG:4326',
    'EPSG:4258': 'EPSG:4258',
  };

  labelFormat = {
    WFS: 'outputFormat',
    WMS: 'FORMAT',
    WCS: 'FORMAT',
  };
  labelLayer = {
    WFS: 'typename',
    WMS: 'layers',
    WCS: 'identifiers',
  };

  inseeLabel: string;

  // Dropdown format
  dropDownFormatsToggle: false;
  selectedFormat: Format;

  // Dropdown projection
  dropDownProjectionToggle: false;
  selectedProjection: {
    name: '',
    bbox: any,
  };
  projectionList;

  // Dropdown insee
  dropDownInseeToggle: false;
  selectedInsee: {
    commune: '',
    insee: '',
  };
  communeInseeList = communeInsee;

  // number of features (for WFS)
  numberFeatures: number = 100;
  inputCountFeatures: number = 100;
  isAllFeaturesChecked = false;

  fromFeature: number = 0;
  minFromFeature: number = 0;

  // Object that will manage the display of the queryable url
  queryableParameters = {
    baseUrl: '',
    baseParameters: '',
    layer: '',
    outputFormat: '',
    outputFormatActive: false,
    projection: '',
    projectionActive: false,
    bbox: '',
    insee: '',
    inseeActive: false,
    numberFeatures: false,
    fromFeature: false,
  };

  constructor(
    private _datasetDetailService: DatasetDetailService,
    private _notificationService: NotificationService,
    private _sanitizer: DomSanitizer,
  ) { }

  // reset the values when the input( selectedResource) is changed
  ngOnChanges() {

    this.queryableParameters = {
      baseUrl: '',
      baseParameters: '',
      layer: '',
      outputFormat: '',
      outputFormatActive: false,
      projection: '',
      projectionActive: false,
      bbox: '',
      insee: '',
      inseeActive: false,
      numberFeatures: false,
      fromFeature: false,
    };

    this.initSettings();
  }

  initSettings() {
    // By default we display for each resource the root URL.
    // Then we check if all necessary information are there to init the custom URL tool
    this.allSettingsOK = true;
    if ((this.resource.formats && this.resource.formats.length < 1) ||
      (this.resource.type === linkFormats.wms && !this.resource.metadataLink.bbox_by_projection) ||
      (this.resource.type === linkFormats.wcs && !this.resource.metadataLink.bbox_by_projection) ||
      this.resource.type === linkFormats.wfs &&
      (!this.resource.metadataLink.projections || this.resource.metadataLink.projections.length < 1)) {
      this.allSettingsOK = false;
    }

    if (this.allSettingsOK) {
      // This is the starting index feature. For WS: 1, WFS: 0
      if (this.resource.type === linkFormats.ws) {
        this.minFromFeature = this.fromFeature = 1;
      } else if (this.resource.type === linkFormats.wfs) {
        this.minFromFeature = this.fromFeature = 0;
      }
      this.isAllFeaturesChecked = false;
      this.inputCountFeatures = 100;
      this.numberFeatures = 100;
      this.initURLCustomization();
    }
  }

  ngOnInit() {
    this.messageClipboard = geosource.mapMessages.share;
    this.messageClipboardRoot = geosource.mapMessages.share;

    this.initSettings();
  }

  initURLCustomization() {
    this.projectionList = [];

    // Set the insee property label for this dataset
    if (this._datasetDetailService.datasetData[0]) {
      const properties = Object.keys(this._datasetDetailService.datasetData[0].properties);
      this.inseeLabel = properties.find((prop) => { return prop.includes('insee'); });
    }

    this.selectedFormat = this.resource.formats[0];
    this.communeInseeList.sort((a, b) => (a.commune > b.commune) ? 1 : -1);

    // Get projections and set the default one (depending on the resource)
    if (this.resource.type === linkFormats.wms) {
      this.projectionList = this.transformObjectToArray(this.resource.metadataLink.bbox_by_projection);
      this.selectedProjection = this.projectionList[0];
    } else if (this.resource.type === linkFormats.wfs) {
      this.resource.metadataLink.projections.forEach((key) => {
        this.projectionList.push({
          name: key,
          bbox: key,
        });
      });
      this.selectedProjection = this.projectionList[0];
    } else if (this.resource.type === linkFormats.wcs) {
      this.projectionList = this.transformObjectToArray(this.resource.metadataLink.bbox_by_projection);
      this.selectedProjection = this.projectionList[0];
    } else if (this.resource.type === linkFormats.ws) {
      this.projectionList = this.transformObjectToArray(this.shpProjections);
      this.selectedProjection = this.projectionList[0];
    }

    // Init the url query parameters that will not be changed by the user
    this.queryableParameters.baseUrl = this.resource.metadataLink.url;
    if (this.resource.type === linkFormats.ws) {
      // Exception for this service, we modify the root URL
      // this.resource.metadataLink.url += '/all.json';
      this.queryableParameters.baseUrl += `/${this.resource.metadataLink.name}/all.json`;
    }
    this.queryableParameters.baseParameters = this.resource.parametersUrl ? `?${this.resource.parametersUrl}` : '';

    if (this.resource.type === linkFormats.kml) {
      this.queryableParameters.baseParameters = `?${this.resource.parametersUrl}` +
        `&typename=${this.resource.metadataLink.name}`;
    }

    this.setProjection(this.selectedProjection);

    if (this.resource.isStandard) {
      this.queryableParameters.layer = `&${this.labelLayer[this.resource.type]}=${this.resource.metadataLink.name}`;
      this.queryableParameters.outputFormat = `&${this.labelFormat[this.resource.type]}=` +
        `${this.selectedFormat.mapServerType}`;
    }

    this.updateQueryableUrl();
    this.updateQueryableurlToDisplay();
  }

  // When the user changes parameters (or when view init), we update the url provided
  // to the <a> link
  updateQueryableUrl() {
    this.queryableUrl = this.queryableParameters.baseUrl +
      this.queryableParameters.baseParameters;
    this.queryableUrl += this.queryableParameters.layer +
      this.queryableParameters.outputFormat +
      this.queryableParameters.insee +
      this.queryableParameters.projection +
      this.queryableParameters.bbox
      ;

    if (this.resource.type === linkFormats.wfs) {
      this.queryableUrl += `&startIndex=${this.fromFeature}`;
    } else if (this.resource.type === linkFormats.ws) {
      if (this.selectedFormat.name === 'JSON') {
        this.queryableUrl += `?maxfeatures=${this.numberFeatures}&start=${this.fromFeature}`;
      } else {
        this.queryableUrl += `&maxfeatures=${this.numberFeatures}&start=${this.fromFeature}`;
      }
    }

    if (!this.isAllFeaturesChecked && this.resource.type === linkFormats.wfs) {
      this.queryableUrl += `&count=${this.numberFeatures}`;
    }

  }

  updateQueryableurlToDisplay() {
    let queryableUrlToDisplay = '';

    queryableUrlToDisplay = this.queryableParameters.baseUrl
      + this.queryableParameters.baseParameters
      + this.queryableParameters.layer;

    // Project & bbox
    if (this.queryableParameters.projectionActive) {
      queryableUrlToDisplay += `<span class="is-active-parameter">${this.queryableParameters.projection}`
        + `${this.queryableParameters.bbox}</span>`;
    } else {
      queryableUrlToDisplay += this.queryableParameters.projection + this.queryableParameters.bbox;
    }

    // Format
    if (this.selectedFormat) {
      if (this.queryableParameters.outputFormatActive) {
        queryableUrlToDisplay +=
          `<span class="is-active-parameter">${this.queryableParameters.outputFormat}</span>`;
      } else {
        queryableUrlToDisplay += this.queryableParameters.outputFormat;
      }
    }

    // Insee
    if (this.selectedInsee) {
      if (this.queryableParameters.inseeActive) {
        queryableUrlToDisplay +=
          `<span class="is-active-parameter">${this.queryableParameters.insee}</span>`;
      } else {
        queryableUrlToDisplay += this.queryableParameters.insee;
      }
    }

    // Number features WFS
    if (!this.isAllFeaturesChecked && this.resource.type === this.linkFormats.wfs) {
      if (this.queryableParameters.numberFeatures) {
        queryableUrlToDisplay +=
          `<span class="is-active-parameter">&count=${this.numberFeatures}</span>`;
      } else {
        queryableUrlToDisplay += `&count=${this.numberFeatures}`;
      }
    }

    // Number features WS
    if (this.resource.type === this.linkFormats.ws) {
      if (this.queryableParameters.numberFeatures) {
        queryableUrlToDisplay += this.selectedFormat.name === 'JSON' ?
          `<span class="is-active-parameter">?maxfeatures=${this.numberFeatures}</span>` :
          `<span class="is-active-parameter">&maxfeatures=${this.numberFeatures}</span>`;
      } else {
        queryableUrlToDisplay += this.selectedFormat.name === 'JSON' ?
          `?maxfeatures=${this.numberFeatures}` :
          `&maxfeatures=${this.numberFeatures}`;
      }
    }

    // From feature WFS
    if (this.resource.type === this.linkFormats.wfs) {
      if (this.queryableParameters.fromFeature) {
        queryableUrlToDisplay +=
          `<span class="is-active-parameter">&startIndex=${this.fromFeature}</span>`;
      } else {
        queryableUrlToDisplay += `&startIndex=${this.fromFeature}`;
      }
    }

    // From feature WS
    if (this.resource.type === this.linkFormats.ws) {
      if (this.queryableParameters.fromFeature) {
        queryableUrlToDisplay +=
          `<span class="is-active-parameter">&start=${this.fromFeature}</span>`;
      } else {
        queryableUrlToDisplay += `&start=${this.fromFeature}`;
      }
    }

    this.queryableUrlToDisplay = this._sanitizer.bypassSecurityTrustHtml(queryableUrlToDisplay);
  }

  // Set the selected format and set the queryable parameters
  // depending on the resource
  setFormat(format: Format) {
    this.selectedFormat = format;
    this.resetUrlHilightedParameters();
    if (this.resource.type !== linkFormats.ws) {
      this.queryableParameters.outputFormat = `&${this.labelFormat[this.resource.type]}=` +
        `${this.selectedFormat.mapServerType}`;
      this.queryableParameters.outputFormatActive = true;
      this.queryableParameters.inseeActive = false;
      if (!this.selectedFormat.isProjectable) {
        this.queryableParameters.insee = '';
      }
    } else {
      // Specific treatment for the WS service. We need manipulate the url
      // depending the json or shapefile format wanted
      if (this.selectedFormat.name === 'JSON') {
        const specificRootURL = this.resource.metadataLink.url.replace('all.json', '');
        this.queryableParameters.baseUrl = `${specificRootURL}/${this.resource.metadataLink.name}/all.json`;
        this.selectedInsee = null;
        this.queryableParameters.insee = '';
        this.queryableParameters.projection = '';
      } else {
        this.queryableParameters.baseUrl = this.resource.metadataLink.url.replace('/all.json', '');
        this.queryableParameters.baseUrl += `/${this.resource.metadataLink.name}.shp?`;
        if (this.selectedFormat.isProjectable) {
          this.setProjection(this.selectedProjection);
        }
      }
    }

    this.updateQueryableUrl();
    this.updateQueryableurlToDisplay();
  }

  // Set the selected insee and set the queryable parameters
  // depending on the resource
  setInsee(insee) {
    this.selectedInsee = insee;
    if (!this.selectedInsee || !this.inseeLabel) {
      this.queryableParameters.insee = '';
    } else {
      this.resetUrlHilightedParameters();
      if (this.resource.type === linkFormats.wfs) {
        this.queryableParameters.insee = '&filter=Filter=<Filter> <PropertyIsLike wildcard="*" ' +
          'singleChar=\'.\' escape=\'_\'> <PropertyName>' +
          `${this.inseeLabel}</PropertyName><Literal>${this.selectedInsee.insee}` +
          '</Literal></PropertyIsLike> </Filter>';
      } else if (this.resource.type === linkFormats.ws) {
        this.queryableParameters.baseUrl = this.resource.metadataLink.name.replace('\/all.json', '');
        const match = (/\/ws\/([^\/]+)/g).exec(this.resource.metadataLink.url);
        const serviceName = match[1] ? match[1] : '';
        this.queryableParameters.insee = `&mask_db=${serviceName}&mask_layer=adr_voie_lieu.adrcommune` +
          `&mask_field=insee&mask_value=${this.selectedInsee.insee}`;
      }
      this.queryableParameters.inseeActive = true;
    }

    this.updateQueryableUrl();
    this.updateQueryableurlToDisplay();
  }

  // Set the selected projection and set the queryable parameters
  // depending on the resource
  setProjection(projection) {
    this.selectedProjection = projection;
    this.resetUrlHilightedParameters();
    if (this.resource.type === linkFormats.wfs) {
      this.queryableParameters.projection = `&SRSNAME=${projection.name}`;
    } else if (this.resource.type === linkFormats.wms || this.resource.type === linkFormats.wcs) {
      this.queryableParameters.projection = `&CRS=${projection.name}`;
      this.queryableParameters.bbox = `&BBOX=${projection.bbox.minx},` +
        `${projection.bbox.miny},` +
        `${projection.bbox.maxx},` +
        `${projection.bbox.maxy}`;
    } else if (this.resource.type === linkFormats.ws) {
      if (this.selectedFormat.name !== 'JSON') {
        this.queryableParameters.projection = `srsname=${projection.name}`;
      }
    }

    this.queryableParameters.projectionActive = true;
    this.updateQueryableUrl();
    this.updateQueryableurlToDisplay();
  }

  setActiveCountFeatures() {
    this.numberFeatures = this.inputCountFeatures;
    this.resetUrlHilightedParameters();
    this.queryableParameters.numberFeatures = true;
    this.updateQueryableUrl();
    this.updateQueryableurlToDisplay();
  }

  setActiveFromFeature() {
    this.resetUrlHilightedParameters();
    this.queryableParameters.fromFeature = true;
    this.updateQueryableUrl();
    this.updateQueryableurlToDisplay();
  }

  checkAllFeatures() {
    this.numberFeatures = this.isAllFeaturesChecked ?
      this._datasetDetailService.dataset.totalData : this.inputCountFeatures;
    this.updateQueryableUrl();
    this.updateQueryableurlToDisplay();
  }

  // Reset the variables used to highlight some parameters in the
  // personnalized url
  resetUrlHilightedParameters() {
    this.queryableParameters.projectionActive = false;
    this.queryableParameters.inseeActive = false;
    this.queryableParameters.outputFormatActive = false;
    this.queryableParameters.fromFeature = false;
    this.queryableParameters.numberFeatures = false;
  }

  transformObjectToArray(object) {
    const array = [];
    const keys = Object.keys(object);
    keys.forEach((key) => {
      array.push({
        name: key,
        bbox: object[key],
      });
    });
    return array;
  }

  getFileName() {
    return `${this.resource.metadataLink.name}.${this.selectedFormat.fileExtension}`;
  }

  // saveFile(url: string) {
  //   saveAs(url, `${this.resource.metadataLink.name}.${this.selectedFormat.fileExtension}`);
  //   this._notificationService.notify(
  //     {
  //       type: 'success',
  //       message: notificationMessages.general.startDownload,
  //     },
  //   );
  // }

  get totalData() {
    return this._datasetDetailService.dataset.totalData;
  }

  getProjectionLabel(name: string) {

    const projection = this.projections.find((projection) => {
      return projection.name === name;
    });
    return projection ? `${projection.commonName} (${projection.name})` : name;
  }

  copyToClipboard(text, isRoot) {
    const selBox = document.createElement('textarea');
    selBox.style.position = 'fixed';
    selBox.style.height = '0';
    selBox.value = text;
    document.body.appendChild(selBox);
    selBox.focus();
    selBox.select();
    document.execCommand('copy');
    document.body.removeChild(selBox);

    if (isRoot) {
      this.messageClipboardRoot = geosource.mapMessages.copied;
      setTimeout(() => {
        this.messageClipboardRoot = geosource.mapMessages.share;
      // tslint:disable-next-line: align
      }, 2000);
    } else {
      this.messageClipboard = geosource.mapMessages.copied;
      setTimeout(() => {
        this.messageClipboard = geosource.mapMessages.share;
      // tslint:disable-next-line: align
      }, 2000);
    }

  }
}
