import { Component, OnDestroy, OnInit } from '@angular/core';
import { Meta } from '@angular/platform-browser';
import { ActivatedRoute, Router, Scroll } from '@angular/router';
import { Subscription } from 'rxjs';
import { filter } from 'rxjs/operators';
import { ViewportScroller } from '../../../../../node_modules/@angular/common';
import { datatsetDataRepresentationType, geosource } from '../../../../i18n/traductions';
import { NavigationHistoryService } from '../../../core/services';
import { AppRoutes } from '../../../routes';
import { IPageHeaderInfo, Metadata, typesMetadata } from '../../../shared/models';
import { DatasetDetailService } from '../../services';

@Component({
  selector: 'app-dataset-detail',
  templateUrl: './dataset-detail.component.html',
  styleUrls: ['./dataset-detail.component.scss'],
})
export class DatasetDetailComponent implements OnInit, OnDestroy {

  AppRoutes = AppRoutes;
  pageHeaderInfo: IPageHeaderInfo = {
    title: '',
  };
  readonly typesMetadata = typesMetadata;
  position: any;
  scrollSubscription: Subscription;
  isLoading = false;

  constructor(
    private _route: ActivatedRoute,
    private _datasetDetailService: DatasetDetailService,
    private _router: Router,
    private _scroller: ViewportScroller,
    private _navigationHistoryService: NavigationHistoryService,
    private _meta: Meta,
  ) { }

  ngOnInit() {
    this.position = [0, 0];
    // Keep scroll position between tabs navigation
    this.scrollSubscription = this._router.events.pipe(
      // Filter on scroll event
      filter(e => e instanceof Scroll),
      // Filter on navigation event children of Approutes.datasets+'/' ('datasets/')
      filter(e => e['routerEvent'].url.search(`${AppRoutes.datasets.uri}/`) !== -1),
    ).subscribe((e) => {
      this._scroller.scrollToPosition(this.position);
    });

    this._route.params.subscribe((params) => {
      // Set the title and description for the research page
      this._meta.updateTag({ name: 'description', content: this.metadata.abstract });

      this.isLoading = true;
      this.initDatasetInfo();
    });
  }

  ngOnDestroy() {
    this.scrollSubscription.unsubscribe();
  }

  get metadata(): Metadata {
    return this._datasetDetailService.datasetMetadata;
  }

  // Number of data found by elasticsearch
  get datasetDataNumber() {
    return this._datasetDetailService.datasetDataNumber;
  }

  get datasetUpdateFrequency() {
    const keyUpdate = this.metadata.updateFrequency ? this.metadata.updateFrequency : 'other';
    return geosource.updateFrequencies[keyUpdate];
  }

  get datasetDataRepresentationType(): string {
    let type = '';
    if (this._datasetDetailService.datasetEditorialMetadata.isPunctual) {
      type = datatsetDataRepresentationType.long.punctual;
    }

    if (this._datasetDetailService.datasetEditorialMetadata.isLinear) {
      type = datatsetDataRepresentationType.long.linear;
    }

    if (this._datasetDetailService.datasetEditorialMetadata.isAreal) {
      type = datatsetDataRepresentationType.long.areal;
    }

    return type;
  }

  get datasetServicesList() {
    const list = this._datasetDetailService.datasetMetadata.link.filter(l => l.service).map(l => l.service);
    let formatted: string;

    if (list.length > 4) {
      formatted = `${list.slice(0, 4).join(', ')} ...`; // Keep the first 4 elements of the list
    } else {
      formatted = list.join(', ');
    }

    return formatted;
  }

  get datasetFormatsList() {

    let list = this._datasetDetailService.datasetMetadata.link.map(l => l.formats);
    // Flat the arrays
    list = [].concat(...list);
    // Remove duplicates
    list = [...new Set(list)];

    let formatted: string;

    if (list.length > 4) {
      formatted = `${list.slice(0, 4).join(', ')} ...`; // Keep the first 4 elements of the list
    } else {
      formatted = list.join(', ');
    }

    return formatted;
  }

  get hasMap() {
    return this._datasetDetailService.dataset.hasMap;
  }

  get hasTable() {
    return this._datasetDetailService.dataset.hasTable;
  }

  initDatasetInfo() {
    this.pageHeaderInfo = {
      title: this.metadata.title,
      metadataSubtitle: this.metadata.subtitle,
      subtitle: this.metadata.contacts.map(c => c.organisationName).join(', '),
    };

    this.isLoading = false;

    // If a tab (info, data...) is not specified in the url then redirect to the data tab if the dataset
    // as a table or a map or to the info tab in the failing case
    if (!this._route.snapshot.firstChild) {
      if (this.hasTable || this.hasMap) {
        this._router.navigate([
          `/${AppRoutes.datasets.uri}/${this._datasetDetailService.dataset.slug}/${AppRoutes.data.uri}`,
          // tslint:disable-next-line: align
        ], { queryParamsHandling: 'preserve' });
      } else {
        this._router.navigate([
          `/${AppRoutes.datasets.uri}/${this._datasetDetailService.dataset.slug}/${AppRoutes.info.uri}`,
        ]);
      }
    } else {
      // Making that the url contains the slug and not the uuid of the dataset
      this._router.navigate([
        // tslint:disable-next-line: max-line-length
        `/${AppRoutes.datasets.uri}/${this._datasetDetailService.dataset.slug}/${this._route.snapshot.firstChild.url[0]}`,
      ],
        // tslint:disable-next-line: align
        { queryParamsHandling: 'preserve' });
    }

    // Emit event to indicate to child component that the dataset has changed
    this._datasetDetailService.datasetChanged();
  }

  setPosition() {
    this.position = this._scroller.getScrollPosition();
  }

  goToPreviousPage() {
    let index = 1; // Start to retrieve the previous element
    let url = this._navigationHistoryService.getFromLast(index);
    // While the previous element still has /jeux-de-donnees or is not null (no previous element)
    // get the previous element of the previous element...
    while (url !== null && url.search(`${AppRoutes.datasets.uri}/`) !== -1) {
      // the method getFromLast deletes every history after the index required therefor as we already entered
      // once in the function we need now to retrieve the last element
      index = 0;
      url = this._navigationHistoryService.getFromLast(index);
    }

    // If url is null then redirect to research page
    if (url == null) {
      url = `/${AppRoutes.research.uri}`;
    }

    this._router.navigateByUrl(url);
  }
}
