import { Component, OnInit } from '@angular/core';
import { CookieService } from 'ngx-cookie-service';
import { Subscription } from 'rxjs';
import { environment } from '../../../../environments/environment';
import { notificationMessages } from '../../../../i18n/traductions';
import { Notification } from '../../../core/models';
import { NotificationService } from '../../../core/services';
import { IMetadataLink, linkFormats, Metadata } from '../../../shared/models';
import { UserService } from '../../../user/services';
import { Resource } from '../../models';
import { DatasetDetailService, ResourcesService } from '../../services';

@Component({
  selector: 'app-dataset-downloads',
  templateUrl: './dataset-downloads.component.html',
  styleUrls: ['./dataset-downloads.component.scss'],
})
export class DatasetDownloadsComponent implements OnInit {

  linkFormats = linkFormats;

  metadata: Metadata;
  sub: Subscription;
  downloadableResources: Resource[];
  staticResources: IMetadataLink[];
  otherResources: IMetadataLink[];

  cguModalIsOpened: boolean;

  isSample = false;
  resources: Resource[];
  hasBeenInitialized = false;

  constructor(
    private _datasetDetailService: DatasetDetailService,
    private _resourcesService: ResourcesService,
    private _notificationService: NotificationService,
    private _cookieService: CookieService,
    private _userService: UserService,

  ) { }

  ngOnInit() {
    this.resources = [];
    this.downloadableResources = [];
    this.staticResources = [];
    this.otherResources = [];

    this.cguModalIsOpened = (this._cookieService.get(environment.cguAcceptedCookieName) !== 'true') &&
      !this._userService.userIsSignedIn;

    this.isSample = this._datasetDetailService.dataset.editorialMetadata.isSample;

    this.initialize();
    this._resourcesService.getResources().subscribe(
      (results) => {
        this.resources = results;

        this.initialize();
        this.sub = this._datasetDetailService.dataset$.subscribe(() => {
          this.initialize();
        });
      },
      (err) => {
        this._notificationService.notify(new Notification({
          type: 'error',
          message: `${notificationMessages.resources.initializationError}`,
        }));
      },
    );
  }

  ngOnDestroy() {
    if (this.sub) {
      this.sub.unsubscribe();
    }
  }

  // Resources are organized on 2 levels :
  // -> 1st level: by metadata link description
  // -> 2nd level: for each link description in 3 categories:
  //    - queryable
  //    - downloadable
  //    - other
  // How to organize: we compare the type of one resource from our resource service
  // and the ES metadata link property. If there is a match, the service will let us know if the resource
  // is queryable or downloadable. If there is no match and the ES resource cannot be found in the service,
  // we put it in "others" resources catagory.
  initialize() {
    this.metadata = this._datasetDetailService.datasetMetadata;

    if (this.metadata && this.resources.length > 0 && !this.hasBeenInitialized) {
      this.hasBeenInitialized = true;
      if (this.metadata.link) {
        this.metadata.link.forEach((link) => {
          if (link.service) {
            this.resources.forEach((resource) => {
              const resourceCopy = Object.assign({}, resource);

              resourceCopy.metadataLink = link;
              if (link.service === resource.type && resourceCopy.isQueryable) {

                // We exclude Shapefile if from WFS. The Shapefile from WS has priority.
                if (resourceCopy.type === this.linkFormats.wfs) {
                  resourceCopy.formats = resourceCopy.formats.filter(f => f.name !== 'ShapeFile');
                }

                // For non-geographical data, the Shapefile is not available for the WS service
                if (resourceCopy.type === 'WS' && this.metadata.type === 'nonGeographicDataset') {
                  resourceCopy.formats = resourceCopy.formats.filter(f => f.name !== 'ShapeFile');
                }

                if (!this.downloadableResources[link.description]) {
                  this.downloadableResources[link.description] = [];
                }
                this.downloadableResources[link.description].push(resourceCopy);
              }
            });
          } else if (link.service === undefined) {
            if (!link.name.includes('Licence')) {
              if (link.formats) {
                if (!this.staticResources[link.description]) {
                  this.staticResources[link.description] = [];
                }
                this.staticResources[link.description].push(link);
              } else {
                this.otherResources.push(link);
              }
            }
          }
        });
      }
    }
  }

  get hasDownloadableResources() {
    return Object.keys(this.downloadableResources).length > 0;
  }

  get hasStaticResources() {
    return Object.keys(this.staticResources).length > 0;
  }

  get hasOtherResources() {
    return Object.keys(this.otherResources).length > 0;
  }

  cguAccepted() {
    this.cguModalIsOpened = false;
  }

}
