import { Component, OnDestroy, OnInit } from '@angular/core';
import { Subscription } from 'rxjs';
import { datatsetDataRepresentationType, geosource } from '../../../../i18n/traductions';
import { AppRoutes } from '../../../routes';
import { IResponsibleParty, Metadata, typesMetadata } from '../../../shared/models';
import { DatasetDetailService } from '../../services';

@Component({
  selector: 'app-dataset-info',
  templateUrl: './dataset-info.component.html',
  styleUrls: ['./dataset-info.component.scss'],
})
export class DatasetInfoComponent implements OnInit, OnDestroy {
  AppRoutes = AppRoutes;
  maxCharactersNumber = 500;
  displayFullAbstract = false;

  metadata: Metadata;
  descriptions: Object[];
  sub: Subscription;

  constructor(
    private _datasetDetailService: DatasetDetailService,
  ) { }

  ngOnInit() {
    // Initilize component variables with value from the service
    this.initializeInfo();

    // Subcribe to changes in the service
    this.sub = this._datasetDetailService.dataset$.subscribe(() => {
      this.initializeInfo();
    });
  }

  ngOnDestroy() {
    this.sub.unsubscribe();
  }

  initializeInfo() {
    this.metadata = this._datasetDetailService.datasetMetadata;
  }

  get parent() {
    return this.metadata.parentDataset;
  }

  get children() {
    return this._datasetDetailService.datasetChildren.data;
  }

  get geographicalInfo(): IGeographicalInfo {
    let info = null;
    if (this.metadata.type === typesMetadata.dataset) {
      info = {
        representationType: this.getDatasetDataRepresentationType(),
        scale: `1:${this.metadata.denominator}`,
        resolution: this.metadata.resolution,
        geographicalExtend: {
          maxEast: `${this.metadata.max_east} est,`,
          maxWest: `${this.metadata.max_west} ouest`,
          maxSouth: `${this.metadata.max_south} sud,`,
          maxNorth: `${this.metadata.max_north} nord,`,
        },
        coordinatesSystem: this.metadata.crs,
      };
    }

    return info;
  }

  get abstract() {
    let value = null;
    if (this.displayFullAbstract || this.metadata.abstract.length <= this.maxCharactersNumber) {
      value = this.metadata.abstract;
    } else {
      value = `${this.metadata.abstract.slice(0, this.maxCharactersNumber)}...`;
    }
    return value;
  }

  toogleAbstractDisplayState() {
    this.displayFullAbstract = !this.displayFullAbstract;
  }

  get displayAbstractShowMoreBtn() {
    return this.metadata.abstract.length > this.maxCharactersNumber && !this.displayFullAbstract;
  }

  get displayAbstractShowLessBtn() {
    return this.metadata.abstract.length > this.maxCharactersNumber && this.displayFullAbstract;
  }

  getDatasetDataRepresentationType(): string {
    let type = '';
    if (this._datasetDetailService.datasetEditorialMetadata.isPunctual) {
      type = datatsetDataRepresentationType.short.punctual;
    }

    if (this._datasetDetailService.datasetEditorialMetadata.isLinear) {
      type = datatsetDataRepresentationType.short.linear;
    }

    if (this._datasetDetailService.datasetEditorialMetadata.isAreal) {
      type = datatsetDataRepresentationType.short.areal;
    }

    return type;
  }

  get summaryInfo(): ISummaryInfo {
    return {
      // tslint:disable-next-line:max-line-length
      isOpenAccess: this._datasetDetailService.datasetEditorialMetadata.isOpenAccess,
      // tslint:disable-next-line:max-line-length
      restriction: this._datasetDetailService.datasetEditorialMetadata.isOpenAccess ? geosource.accessRestriction.open : geosource.accessRestriction.restricted,
      publicationDate: this.metadata.publicationDate,
      viewsNumber: this._datasetDetailService.datasetNumberOfViews,
      updateFrequency: this.metadata.updateFrequency ?
        geosource.updateFrequencies[this.metadata.updateFrequency] : geosource.updateFrequencies.other,
      dataType: geosource.datasetTypes[this.metadata.type],
    };
  }

  get producers(): IResponsibleParty[] {
    return this.metadata.contacts;
  }

  get categories() {
    return this.metadata.getCategories().join(', ');
  }

  get licenceUrl(): string {
    const license = this.metadata.link.find(l => l.name.includes('Licence'));
    const url = license && license.url ? license.url : null;
    return url;
  }

  get keywords() {
    return this.metadata.keyword.join(', ');
  }
}

interface IGeographicalInfo {
  representationType: string;
  scale: string;
  resolution: string;
  geographicalExtend: {
    maxEast: string;
    maxSouth: string;
    maxNorth: string;
    maxWest: string;
  };
  coordinatesSystem: string;
}

interface ISummaryInfo {
  isOpenAccess: boolean;
  restriction: string;
  publicationDate: string;
  viewsNumber: number;
  updateFrequency: string;
  dataType: string;
}
