import { DatePipe, ViewportScroller } from '@angular/common';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { Meta } from '@angular/platform-browser';
import { Subscription } from 'rxjs';
import { metaDescription } from '../../../../i18n/traductions';
import { Notification } from '../../../core/models';
import { NotificationService } from '../../../core/services';
import { MatomoService } from '../../../core/services/matomo.service';
import { CMSContent } from '../../../editorialisation/models';
import { IScope } from '../../../elasticsearch/models';
import { AppRoutes } from '../../../routes';
import { Dataset, IMetadataLink, PaginatorOptions } from '../../../shared/models';
import { SearchSuggestion } from '../../models';
import { DatasetResearchService } from '../../services';

@Component({
  selector: 'app-results',
  templateUrl: './results.component.html',
  styleUrls: ['./results.component.scss'],
})
export class ResultsComponent implements OnInit, OnDestroy {

  AppRoutes = AppRoutes;

  results: Dataset[] | CMSContent[];

  countResults: [];

  searchChangeSub: Subscription;
  // Paginator options
  paginator: PaginatorOptions;
  displayFilters = false;
  displaySuggestion = false;
  suggestion = new SearchSuggestion({});

  selectedScope: IScope;

  constructor(
    private _datasetResearchService: DatasetResearchService,
    private _notificationService: NotificationService,
    private _datePipe: DatePipe,
    private _matomoService: MatomoService,
    private _viewportScroller: ViewportScroller,
    private _meta: Meta,
  ) { }

  ngOnInit() {
    // Set the title and description for the research page
    this._meta.updateTag({ name: 'description', content: metaDescription.research });

    this.countResults = [];
    this.paginator = {
      pageIndex: this._datasetResearchService.pageIndex + 1,
      length: 0,
      pageSize: this._datasetResearchService.pageSize,
      pageSizeOptions: [5, 10, 20],
    };
    this.search();
    this.getSuggestion();
    this.searchChangeSub = this._datasetResearchService.searchChange$.subscribe(
      (value) => {
        this.search();
        this.getSuggestion();
      },
    );
  }

  ngOnDestroy() {
    this.searchChangeSub.unsubscribe();
  }

  search() {
    this._datasetResearchService.getResults().subscribe(
      (res) => {
        this.selectedScope = this._datasetResearchService.scopeReasearch;
        const position = this._viewportScroller.getScrollPosition();
        this.results = res;

        this.countResults = [];
        this._datasetResearchService.resultsCount.forEach((item) => {
          this.countResults[item.scopeKey] = item.count || 0;
        });

        this.paginator.pageSize = this._datasetResearchService.pageSize;
        this.paginator.pageIndex = this._datasetResearchService.pageIndex + 1;

        // Set pagination depending on the selected scope
        this.paginator.length = this.countResults[this.selectedScope.key];
        this._viewportScroller.scrollToPosition(position);

      },
      (err) => {
        this._notificationService.notify(new Notification({
          message: err.message,
          type: 'error',
        }));
      },
    );

  }

  // When pagination is changed by user, we update results with new pagination options
  changePagination(pageIndex) {
    this._datasetResearchService.shouldAggregateResultCount = false;
    this._datasetResearchService.shouldAggregateFilters = false;
    this._datasetResearchService.paginationChanged(this.paginator.pageSize, pageIndex - 1);
  }

  changePageSize(pageSize) {
    this._datasetResearchService.paginationChanged(pageSize, 0);
  }

  formatFoundItem(e) {
    return `<u>${e.label}:</u>${e.content}`;
  }

  formatProtocols(uriList: IMetadataLink[]): string {
    return uriList.map((uri) => {
      return uri.service;
    }).join(', ');
  }

  toggleFilters() {
    this.displayFilters = !this.displayFilters;
  }

  getSuggestion() {
    this._datasetResearchService.getSuggestion(this._datasetResearchService.searchString).subscribe(
      (suggestion) => {
        this.suggestion = suggestion;
        if (this.suggestion.text !== '' && this.suggestion.text !== null && this.suggestion.text !== undefined) {
          this.displaySuggestion = true;
        } else {
          this.displaySuggestion = false;
        }
      },
      (err) => {
        this.displaySuggestion = false;
      },
    );
  }

  scopeChanged(value: IScope) {
    this.selectedScope = value;
    this._datasetResearchService.scopeChanged(this.selectedScope);
  }

  useSuggestion() {
    this._datasetResearchService.searchChanged(this.suggestion.text);
  }

  formatDate(date: string, format: string) {
    try {
      let res = this._datePipe.transform(date, format);
      if (!res) {
        res = '-';
      }
      return res;
    } catch (error) {
      return '-';
    }
  }

  isDataset(value: Dataset | CMSContent) {
    return value instanceof Dataset;
  }

  get isLoading() {
    return this._datasetResearchService.isLoading;
  }

  get fromResultNumber() {
    return ((this.paginator.pageIndex - 1) * this.paginator.pageSize) + 1;
  }

  get toResultNumber() {
    return Math.min(((this.paginator.pageIndex - 1) * this.paginator.pageSize) + 10, this.paginator.length);
  }
}
