import { Component, OnInit } from '@angular/core';
import { Meta } from '@angular/platform-browser';
import { Router } from '@angular/router';
import { Angulartics2Piwik } from 'angulartics2/piwik';
import { forkJoin } from 'rxjs/internal/observable/forkJoin';
import { environment } from '../../../../environments/environment';
import { metaDescription, notificationMessages } from '../../../../i18n/traductions';
import { Notification } from '../../../core/models';
import { NotificationService } from '../../../core/services';
import { DatasetResearchService } from '../../../datasets/services';
import { ElasticsearchService } from '../../../elasticsearch/services/elasticsearch.service';
import { AppRoutes } from '../../../routes';
import { scopesResearch } from '../../../shared/variables';
import { CMSContent, IESCMSResponse } from '../../models/cms-content.model';

@Component({
  selector: 'app-home',
  templateUrl: './home.component.html',
  styleUrls: ['./home.component.scss'],
})
export class HomeComponent implements OnInit {

  AppRoutes = AppRoutes;
  environment = environment;
  notificationMessages = notificationMessages;

  triggerEasterEgg: boolean = false;
  numberOfClickEgg: number = 0;

  latestPosts: CMSContent[];

  defaultImage: string = 'assets/img/home_image_garbage.jpg';
  finalImage: string = 'assets/img/home_image.jpg';

  constructor(
    private _elasticsearchService: ElasticsearchService,
    private _datasetResearchService: DatasetResearchService,
    private _notificationService: NotificationService,
    private _router: Router,
    private _angulartics2Piwik: Angulartics2Piwik,
    private _meta: Meta,
  ) { }

  ngOnInit() {
    // Set the title and description for the home page
    this._meta.updateTag({ name: 'description', content: metaDescription.home });

    this._datasetResearchService.resetResearch(false);
    this.latestPosts = [];

    // We get the 3 latests posts.
    // Then we check if one post is prioritary. 2 possibilities
    // - one prioritary + 2 latest posts
    // - 3 latest posts
    const optionsLatest = {
      size: 3,
      sortDate: true,
      status: 'published',
      type: 'post',
    };
    const optionSticky = {
      size: 1,
      featured: true,
      type: 'post',
    };

    forkJoin([
      this._elasticsearchService.getPosts(optionsLatest),
      this._elasticsearchService.getPosts(optionSticky),
    ]).subscribe(
      (results) => {
        const posts = results[0];
        const featuredPost = results[1];

        if (featuredPost.hits.hits && featuredPost.hits.hits.length > 0) {
          const source = featuredPost.hits.hits[0]._source as IESCMSResponse;
          this.latestPosts.push(new CMSContent(source));
        }

        posts.hits.hits.forEach((result) => {
          const source = result._source as IESCMSResponse;
          if (this.latestPosts.length < 3) {
            if (this.latestPosts.length === 0 || source.uuid !== this.latestPosts[0].uuid) {
              const cmsPost = new CMSContent(source);
              this.latestPosts.push(cmsPost);
            }
          }
        });
      },
      (err) => {
        this._notificationService.notify(new Notification({
          type: 'error',
          message: `${notificationMessages.edito.getPosts}`,
        }));
      },
    );
    this._elasticsearchService.getPosts(optionsLatest).subscribe((posts) => {

    });

  }

  // Update the research service with the scope selected, and set the string value to ''.
  // Then navigate to the results view.
  goToPostsResearch() {
    this._datasetResearchService.resetResearch();
    this._datasetResearchService.scopeReasearch = scopesResearch.post;
    this._router.navigate(['/', AppRoutes.research.uri]);
  }

  trackExploreButtonEvent() {
    this._angulartics2Piwik.eventTrack('ExploreButton', { category: 'HomePage', label: 'Explore', value: 1 });
  }
}
