import { geosource } from '../../../i18n/traductions';
import { scopesResearch } from '../../shared/variables';
import { Filter } from './filter.model';
import { ISortOption } from './sort-option.model';

export interface IPostsESOptions {
  type?: string;
  id?: string;
  slug?: string;
  content?: string;
  status?: string;
  shouldHighlight?: boolean;
  size?: number;
  sortDate?: boolean;
  includeStaticContent?: boolean;
  featured?: boolean;
}

export interface IScope {
  key: string;
  label: string;
  elasticType: string[];
  errorItem?: string;
  placeholder?: string;
}

export interface IElasticsearchOptions {
  pageIndex?: number;
  from?: number;
  pageSize?: number;
  sortOptions?: ISortOption;
  searchString?: string;
  filters?: Filter[];
  shouldAggregateResultCount: boolean;
  shouldAggregateFilters: boolean;
  scope: IScope;
  useCache: boolean;
}

export class ElasticsearchOptions {
  _pageIndex: number;
  from?: number;
  _pageSize: number;
  _sortOptions: ISortOption;
  _searchString: string;
  _filters: Filter[];
  shouldAggregateResultCount: boolean;
  shouldAggregateFilters: boolean;
  fromAutocompletion: boolean;
  scope: IScope;
  useCache: boolean;
  otherLicenceAggregations: string[];

  constructor(data?) {
    this.from = (data && data.from != null) ? data.from : 0;
    this._pageSize = (data && data.pageSize != null) ? data.pageSize : 5;
    this._pageIndex = (data && data.pageIndex != null) ? data.pageIndex : 0;
    this._searchString = (data && data.searchString != null) ? data.searchString : '';
    this.shouldAggregateResultCount = true;
    this.shouldAggregateFilters = true;
    this.fromAutocompletion = false;
    this.scope = (data.scope) ? data.scope : scopesResearch.all;
    this.otherLicenceAggregations = [];
    this.useCache = (data && data.useCache != null) ? data.useCache : true;

    // Need to be done in order to avoid object reference copy
    this._sortOptions = {
      value: 'relevance',
      order: 'desc',
    };
    if (data && data.sortOptions != null) {
      this._sortOptions.value = data.sortOptions.value;
      this._sortOptions.order = data.sortOptions.order;
    }
    this._filters = (data && data.filters != null) ? data.filters : [
      new Filter({
        field: 'metadata-fr.category',
        label: geosource.filterCategories.categories,
        aggregations: [],
        type: 'string',
        index: 'dataset',
      }),
      new Filter({
        field: 'metadata-fr.responsibleParty.organisationName',
        subField: 'metadata-fr.responsibleParty.individualName',
        nestedPath: 'metadata-fr.responsibleParty',
        label: geosource.filterCategories.dataProducers,
        aggregations: [],
        type: 'string',
        index: 'dataset',
      }),
      new Filter({
        field: 'metadata-fr.type',
        label: geosource.filterCategories.type,
        aggregations: [],
        type: 'string',
        index: 'dataset',
      }),
      new Filter({
        field: 'metadata-fr.license',
        label: geosource.filterCategories.licences,
        aggregations: [],
        type: 'string',
        index: 'dataset',
      }),
      new Filter({
        field: 'metadata-fr.link.service',
        label: geosource.filterCategories.services,
        aggregations: [],
        type: 'string',
        index: 'dataset',
      }),
      new Filter({
        field: 'metadata-fr.link.formats',
        label: geosource.filterCategories.formats,
        aggregations: [],
        type: 'string',
        index: 'dataset',
      }),
      new Filter({
        field: 'metadata-fr.publicationDate',
        label: geosource.filterCategories.publicationYear,
        aggregations: [],
        type: 'date',
        index: 'dataset',
      }),
      new Filter({
        field: 'metadata-fr.updateFrequency',
        label: geosource.filterCategories.updateFrequency,
        aggregations: [],
        type: 'string',
        index: 'dataset',
      }),
    ];
  }

  // Known licences
  get knownLicences() {
    return ['Licence Ouverte', 'Licence ODbL', 'Licence de réutilisation des données d\'intérêt général'];
  }

  // Getters

  get pageIndex(): number {
    return this._pageIndex;
  }

  get pageSize(): number {
    return this._pageSize;
  }

  get sortOptions(): ISortOption {
    return this._sortOptions;
  }

  get searchString(): string {
    return this._searchString;
  }

  get filters(): Filter[] {
    return this._filters;
  }

  // Setter

  set pageIndex(page: number) {
    this._pageIndex = page;
  }

  set pageSize(size: number) {
    this._pageSize = size;
  }

  set sortOptions(options: ISortOption) {
    this._sortOptions = options;
  }

  set searchString(value: string) {
    this._searchString = value;
  }

  set filters(filters: Filter[]) {
    this._filters = filters;
  }
}

export interface ICountScope {
  scopeKey: string;
  count: number;
}
