import { Injectable } from '@angular/core';
import centroid from '@turf/centroid';
import * as mapboxgl from 'mapbox-gl';
import { fromEvent, Observable, Subject, Subscription } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
import { notificationMessages } from '../../../i18n/traductions.fr';
import { Notification } from '../../core/models';
import { NotificationService } from '../../core/services';
import { APP_CONFIG } from '../../core/services/app-config.service';
import { MapOptions } from '../models/map-options';

@Injectable()
export class MapService {

  private _map: mapboxgl.Map;
  private url: string;
  selectedBaseLayer: any;
  mapOptions: MapOptions;

  eventPopupAdded = false;
  mapIsConstructed: boolean = false;

  // Map features colors
  featureColor: string = '#1d92ff';
  featureHoverColor: string = '#E19190';
  featureHighlightedColor: string = '#da322f'; // Tomato color

  hoveredFeatureId: string;
  highlightedFeatureId: string;

  private _mapSubject = new Subject<any>();
  private _errorSubscription: Subscription;

  selectedFeature; // Contains the gid of the selected feature

  // Properties used to send information to the component
  // about the features state (need to update or not)
  private _mapToUpdate = new Subject<any>();
  private _featureSelected = new Subject<mapboxgl.MapboxGeoJSONFeature>();

  // Mapbox expression. Is set differently depending the data type (circle, fill, line)
  private paintExpression = {};
  private paintColorPropertyName = '';

  constructor(
    private _notificationService: NotificationService,
  ) { }

  // Init the map with basic options for controls, transform request etc...
  createMap(
    mapOptions: MapOptions,
    url: string, baseLayer: any, options?: mapboxgl.MapboxOptions): mapboxgl.Map {
    this.mapOptions = mapOptions;
    this.url = url;

    // Reset to false in ordre to set event listener
    this.eventPopupAdded = false;

    this.selectedBaseLayer = baseLayer;

    options.transformRequest = (url, resourceType) => {
      if (resourceType === 'Tile' && url.startsWith(this.mapOptions.mvtUrl)) {
        return {
          url,
          credentials: 'include',
        };
      }
    };

    this._map = new mapboxgl.Map(options);

    const scale = new mapboxgl.ScaleControl({
      maxWidth: 80,
      unit: 'metric',
    });
    this._map.addControl(scale, 'bottom-right');

    const nav = new mapboxgl.NavigationControl();
    this._map.addControl(nav, 'bottom-right');

    // Create an observable from the map error event and only emit after 1s in order
    // to avoid to many error at the same time
    const errorObservable = fromEvent(this._map, 'error').pipe(debounceTime(1000));

    // Subscribe to the error observable and send a notification
    this._errorSubscription = errorObservable.subscribe((v) => {
      console.log(v);
      this._notificationService.notify(
        new Notification({
          message: notificationMessages.geosource.mapError,
          type: 'error',
        }),
      );
    });

    this._mapSubject.next();

    this.mapIsConstructed = true;

    return this._map;
  }

  // This adds 2 layers:
  // - a WMS layer to display the visual part of the features (WMS service send a png or jpeg)
  // - a data layer, created from a geojson or an MVT service. It is used for the features interaction (hover, click)
  addLayers() {

    this.addWMSLayer();

    // Add the 3d source. Constructed with MVT tiles from the 'fpc_fond_plan_communaut.fpctoit' dataset
    const domain = this.mapOptions.vectorService.url.split('wfs')[1];
    const url = `${this.mapOptions.mvtUrl}${domain}?LAYERS=
fpc_fond_plan_communaut.fpctoit&map.imagetype=mvt&tilemode=gmap&tile={x}+{y}+{z}&mode=tile`;
    this._map.addSource('3d-source', {
      type: 'vector',
      tiles: [url],
    });

    // Add the data layer only if it comes from the Lyon Metropole
    if (this.mapOptions.rasterService.url.includes(APP_CONFIG.backendUrls.wms)) {
      // There is two ways to add tha data layers: from a geojson or from a MVT url
      if (!this.mapOptions.isMVT) {
        if (this.mapOptions.geojson) {
          this._map.addSource('vector-source', {
            type: 'geojson',
            data: this.mapOptions.geojson,
          });
          this.addDataLayer();
        }
      } else {
        if (this.mapOptions.vectorService && this.mapOptions.vectorService.url) {
          const domain = this.mapOptions.vectorService.url.split('wfs')[1];
          const url = `${this.mapOptions.mvtUrl}${domain}?LAYERS` +
            `=${this.mapOptions.vectorService.name}&map.imagetype=mvt&tilemode=gmap&tile={x}+{y}+{z}&mode=tile`;

          this._map.addSource('vector-source', {
            type: 'vector',
            tiles: [url],
          });

          this.addDataLayer();
        }
      }
    }
  }

  addClickEventOnLayer(layer: string) {
    this._map.on('click', layer, (e) => {
      // Set a variable that is verified on the map click event (evey click not only layers)
      // when set to tru it will prevent the action on the map click event to be executed
      e.stopPropagation = true;

      this.selectedFeature = e.features[0].id;

      // Reset the hover and highglithed state for the current and previous feature
      this.changeFeatureState(this.hoveredFeatureId, { highlight: false });
      this.changeFeatureState(this.highlightedFeatureId, { highlight: false });

      // Set highlited style for the current feature
      this.highlightedFeatureId = this.selectedFeature;
      this.changeFeatureState(this.highlightedFeatureId, { highlight: true });

      if (e.point.x > (this._map.getCanvas().width - 400)) {
        // If the screen is not mobile the dataset data details panel push the map that is then smaller
        // therefore we make sure the point/surface/line is still in the view box by recentering the map on the point
        // 768 correspond to 768px which is the mobile breakpoint defined by Bulma our css framework
        if (window.innerWidth > 768) {
          // 70 is the width of the dataset data details panel divided by 4 (by 2 because the map and the table
          // share the remaining space and again by 2 because we want to center the point so there is half on the
          // left, half on the right)
          this._map.panTo(e.lngLat, {
            duration: 500,
            // 70 is the width of the dataset data details panel divided by 4 (by 2 because the map and the table
            // share the remaining space and again by 2 because we want to center the point so there is half on the
            // left, half on the right)
            offset: new mapboxgl.Point(-70, 0),
          },
          );
        }
      }

      this._featureSelected.next(e.features[0]);
    });
  }

  // Change the state of one feature.
  // State has the following format:{state: value}
  // Ex: {hover: true, highlight: false}
  changeFeatureState(featureId: string, state) {
    let stateSource = {} as mapboxgl.FeatureIdentifier;
    if (!this.mapOptions.isMVT) {
      stateSource = {
        source: 'vector-source',
        id: featureId,
      };
    } else {
      stateSource = {
        source: 'vector-source',
        sourceLayer: this.mapOptions.vectorService.name,
        id: featureId,
      };
    }
    if (featureId) {
      const oldState = this._map.getFeatureState(stateSource);

      this._map.setFeatureState(
        stateSource,
        { ...oldState, ...state });
    }
  }

  // Add the raster (WMS) layer
  addWMSLayer() {
    this._map.addSource('wms-source', {
      type: 'raster',
      tiles: [
        `${this.mapOptions.rasterService.url}?bbox={bbox-epsg-3857}&format=image/png&service=WMS&version=1.1.1&request=GetMap&` +
        `srs=EPSG:3857&width=256&height=256&transparent=true&layers=${this.mapOptions.rasterService.name}`,
      ],
      tileSize: 256,
    });

    this._map.addLayer({
      id: 'wms-layer',
      type: 'raster',
      source: 'wms-source',
      paint: {},
    });

  }

  // Add the data layer (from geojson or MVT)
  addDataLayer() {

    let layerOptions = {};

    // Set the paint options depending the geometry type
    // For 'Polygon' and 'MultiPolygon' features
    if (this.mapOptions.dataType.isAreal) {
      this.paintExpression = {
        'fill-color': ['case',
          ['boolean', ['feature-state', 'highlight'], false],
          this.featureHighlightedColor,
          ['boolean', ['feature-state', 'hover'], false],
          this.featureHoverColor,
          'transparent',
        ],
        'fill-opacity': 0.7,
        'fill-outline-color': 'transparent',
      };
      this.paintColorPropertyName = 'fill-color';

      layerOptions = {
        type: 'fill',
        paint: this.paintExpression,
      };
    }

    // For 'Line' and 'MultiLine' features
    if (this.mapOptions.dataType.isLinear) {

      this.paintExpression = {
        'line-color': ['case',
          ['boolean', ['feature-state', 'highlight'], false],
          this.featureHighlightedColor,
          ['boolean', ['feature-state', 'hover'], false],
          this.featureHoverColor,
          'transparent',
        ],
        'line-width': [
          'interpolate', ['linear'], ['zoom'],
          12, 2,
          13, 4,
        ],
        'line-opacity': 0.8,
      };
      this.paintColorPropertyName = 'line-color';

      layerOptions = {
        type: 'line',
        paint: this.paintExpression,
      };
    }

    // For "Point" features
    if (this.mapOptions.dataType.isPunctual) {

      // Set the color expression
      this.paintExpression = {
        'circle-radius': ['case',
          ['boolean', ['feature-state', 'highlight'], false],
          2.5,
          5,
        ],
        'circle-color': ['case',
          ['boolean', ['feature-state', 'highlight'], false],
          'white',
          ['boolean', ['feature-state', 'hover'], false],
          this.featureHoverColor,
          'transparent',
        ],
        'circle-stroke-width': ['case',
          ['boolean', ['feature-state', 'highlight'], false],
          5,
          0,
        ],
        'circle-stroke-color': ['case',
          ['boolean', ['feature-state', 'highlight'], false],
          this.featureHighlightedColor,
          ['boolean', ['feature-state', 'visited'], false],
          'white',
          'transparent',
        ],
      };
      this.paintColorPropertyName = 'circle-color';

      layerOptions = {
        type: 'circle',
        paint: this.paintExpression,
      };
    }

    this._map.addLayer(
      {
        id: 'data-layer',
        source: 'vector-source',
        ...layerOptions,
        ...(this.mapOptions.isMVT ? { 'source-layer': this.mapOptions.vectorService.name } : {}), // if from MVT, this property is needed
      });

    // If not already done, add all the events listeners
    if (!this.eventPopupAdded) {

      // Manage the cursor and feature state for data layer when mouse events
      this._map.on('mousemove', 'data-layer', (e) => {
        this.manageFeatureOnMouseMove(e.features);
      });
      this._map.on('mouseleave', 'data-layer', (e) => {
        this.manageFeatureOnMouseEnd();
      });
      this.addClickEventOnLayer('data-layer');

      // [WARNING] The click event on the map has to be declared after the click event on layers
      // in order for the stopPropagation to work properly
      this._map.on('click', (e) => {
        // If the click was on a layer the stopPropagation property is set
        // we only want to execute the code when the click was not on a feature of a layer
        if (!e.stopPropagation) {
          this.resetSelectedState();
          setTimeout(
            () => {
              const nw = this._map.getBounds().getNorthWest();
              this._map.resize();
              const newNw = this.map.getBounds().getNorthWest();
              const currentCenter = this._map.getCenter();
              const shiftVector = { x: newNw.lng - nw.lng, y: newNw.lat - nw.lat };
              const newCenter = new mapboxgl.LngLat(
                currentCenter.lng - shiftVector.x,
                currentCenter.lat - shiftVector.y,
              );
              this._map.setCenter(newCenter);
            },
            600,
          );
        }
      });
      this.eventPopupAdded = true;
    }
  }

  manageFeatureOnMouseMove(features: any) {
    if (features.length > 0) {
      this._map.getCanvas().style.cursor = 'pointer';
      if (this.hoveredFeatureId) {
        this.changeFeatureState(this.hoveredFeatureId, { hover: false });
      }

      this.hoveredFeatureId = features[0].id;

      this.changeFeatureState(this.hoveredFeatureId, { hover: true });
    }
  }

  manageFeatureOnMouseEnd() {
    if (this.hoveredFeatureId) {
      this._map.getCanvas().style.cursor = '';
      this.changeFeatureState(this.hoveredFeatureId, { hover: false });
    }
    this.hoveredFeatureId = null;
  }

  destroyMap() {
    this._map.remove();
    this._errorSubscription.unsubscribe();
  }

  switch3DLayer() {
    if (!this._map.getLayer('3d-layer')) {

      this._map.addLayer(
        {
          id: '3d-layer',
          type: 'fill-extrusion',
          source: '3d-source',
          'source-layer': 'fpc_fond_plan_communaut.fpctoit',
          paint: {
            'fill-extrusion-color': '#E0E4EF',
            'fill-extrusion-height': {
              property: 'htotale',
              type: 'identity',
            },
            'fill-extrusion-opacity': 0.7,
          },
        });
    } else {
      this._map.removeLayer('3d-layer');
    }
  }

  // Generate the url with map options
  // http://hostName#zoom/latitude/longitude/bearing/pitch/selectedBaseLayerId
  getPermalinkUrl() {
    return `${this.url}?map=${this._map.getZoom()}/${this._map.getCenter().lat}/${this._map.getCenter().lng}` +
      `/${this._map.getBearing()}/${this._map.getPitch()}/${this.selectedBaseLayer.id}`;
  }

  switchLayer(baseLayer) {
    // Set selected base layer
    this.selectedBaseLayer = baseLayer;
    // Change map baseLayer with the new file
    if (this._map.getZoom() > this.selectedBaseLayer.maxzoom) {
      this._map.setZoom(this.selectedBaseLayer.maxzoom - 1);
    }
    this._map.setMaxZoom(this.selectedBaseLayer.maxzoom);

    this._map.setStyle(this.selectedBaseLayer.urlJson);
  }

  setSelectedFeature(selectedFeature) {
    if (selectedFeature && this.selectedFeature !== selectedFeature.id) {
      this.selectedFeature = selectedFeature.id;

      // Reset the hover and highglithed state for the current and previous feature
      this.changeFeatureState(this.hoveredFeatureId, { highlight: false });
      this.changeFeatureState(this.highlightedFeatureId, { highlight: false });

      // Set highlited style for the current feature
      this.highlightedFeatureId = this.selectedFeature;
      this.changeFeatureState(this.highlightedFeatureId, { highlight: true });

      const pointCenter = selectedFeature.geometry.type === 'Point' ?
        selectedFeature.geometry.coordinates : centroid(selectedFeature).geometry.coordinates;
      this._map.panTo(pointCenter, {
        duration: 500,
        // 70 is the width of the dataset data details panel divided by 4 (by 2 because the map and the table
        // share the remaining space and again by 2 because we want to center the point so there is half on the
        // left, half on the right)
        offset: new mapboxgl.Point(-70, 0),
      },
      );

    }
  }

  /*
  * When the search value has been changed, we add a text expression
  * that filters the features containing this text value in at least one of its properties.
  * If there is a match we:
  * - decrease the opacity for the raster layer (WMS) which displays all the features
  * - show the found features from our data layer (WMT or GeoJSON)
  */
  filterBySearchValue(searchValue: string, properties: string[]) {

    if (searchValue) {

      const escapedSearchString = searchValue.replace(/[\=~><\"\?^\${}\(\)\|\&\:\!\/[\]\\]/g, '\\$&');
      const words = escapedSearchString.split(/\s+/);

      // Some basic explanations for the operators expression we use:
      // - "all": returns true if all the conditions are true
      // - "any": returns true if one of the the conditions are true
      // - "in":  can be used in many context, here it looks for a substring in a string. To make it case insensitive, set
      // both the search value and the property value to uppercase with the operator 'upcase'.
      // To learn more about it: https://docs.mapbox.com/mapbox-gl-js/style-spec/#expressions
      // Here is the format of the expression, for example with 2 words:
      // ['all',
      //   ['any',
      //      ['in', ['upcase', 'word1'], ['upcase', ['to-string', ['get', property1]]]],
      //      ['in', ['upcase', 'word1'], ['upcase', ['to-string', ['get', property2]]]],
      //      ['in', ['upcase', 'word1'], ['upcase', ['to-string', ['get', property3]]]]
      //   ],
      //   ['any',
      //      ['in', ['upcase', 'word2'], ['upcase', ['to-string', ['get', property1]]]],
      //      ['in', ['upcase', 'word2'], ['upcase', ['to-string', ['get', property2]]]],
      //      ['in', ['upcase', 'word2'], ['upcase', ['to-string', ['get', property3]]]]
      //   ],
      // ]
      const anyFilter = [];
      words.forEach((word) => {
        const propertiesFilters = [];
        properties.forEach((property) => {
          propertiesFilters.push(['in', ['upcase', word], ['upcase', ['to-string', ['get', property]]]]);
        });
        anyFilter.push(['any', ...propertiesFilters]);
      });

      // Once this search filter expression is done, we add it to the existing paint options (for hover, selected)
      // of the data-layer
      const copyPaintOptions = [... this._map.getPaintProperty('data-layer', this.paintColorPropertyName)];
      copyPaintOptions.splice(copyPaintOptions.length - 1, 0, ['all', ...anyFilter], this.featureHighlightedColor);

      this._map.setFilter('data-layer', ['all', ...anyFilter]);
      this._map.setPaintProperty('data-layer', this.paintColorPropertyName, copyPaintOptions);
      this._map.setPaintProperty('wms-layer', 'raster-opacity', 0.3);

    } else {
      // If value is empty, remove the search expression and set the opacity for the raster layer back to 1.
      this._map.setFilter('data-layer', null);
      this._map.setPaintProperty('data-layer', this.paintColorPropertyName, this.paintExpression[this.paintColorPropertyName]);
      this._map.setPaintProperty('wms-layer', 'raster-opacity', 1);

    }
  }

  get map$(): Observable<void> {
    return this._mapSubject.asObservable();
  }

  get map() {
    return this._map;
  }

  resetSelectedState() {
    this.changeFeatureState(this.highlightedFeatureId, { highlight: false });
    this.selectedFeature = null;
    this.highlightedFeatureId = null;
    this._featureSelected.next(null);
  }

  get featuresToUpdate$(): Observable<void> {
    return this._mapToUpdate.asObservable();
  }

  get featureSelected$(): Observable<any> {
    return this._featureSelected.asObservable();
  }

}
