import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable, Subject } from 'rxjs';
import { IUserInfo, PasswordUpdateForm, User, ILoginResponse, LegacyAccount, IPasswordForgottenForm } from '../models';
import { map, mergeMap, tap } from 'rxjs/operators';
import * as NodeRSA from 'node-rsa';
import { APP_CONFIG } from '../../core/services/app-config.service';

@Injectable()
export class UserService {

  private _user: User = null;
  _userStatusChangedSubject: Subject<boolean>;

  constructor(
    private _http: HttpClient,
  ) {
    this._userStatusChangedSubject = new Subject<boolean>();
  }

  initializeService() {
    return this.getUserInfo().pipe(
      map((userInfo) => {
        this._user = new User(userInfo);
      }),
    );
  }

  // Function and helpers allowing the management of the user session (jwt), info...
  setSession(authResult): boolean {
    let success = false;
    if (authResult && authResult.userInfo) {
      this._user = new User(authResult.userInfo);
      success = true;
      this._userStatusChangedSubject.next(true);
    } else {
      this.resetAuth();
    }

    return success;
  }

  resetAuth() {
    this.logout().subscribe(() => {
      this._user = null;
      this._userStatusChangedSubject.next(false);
    });
  }

  get user() {
    return this._user;
  }

  get userIsSignedIn() {
    return this._user != null;
  }

  get userStatusChanged$(): Observable<boolean> {
    return this._userStatusChangedSubject.asObservable();
  }

  // HTTP Calls
  login(loginForm): Observable<boolean> {
    // Make sure not to mofidy the object passed in parameter (object reference) when encrypting the password
    const form = Object.assign({}, loginForm);
    return this.getPublicKey().pipe(
      tap(
        (publicKey) => {
          form.password = this.encrypt(form.password, publicKey);
        },
      ),
      mergeMap(() => {
        return this._http.post<ILoginResponse>(
          `${APP_CONFIG.backendUrls.auth}login/legacy`,
          form,
          { withCredentials: true },
        );
      }),
      map(
        (res) => {
          return this.setSession(res);
        },
      ),
    );
  }

  logout(): Observable<boolean> {
    return this._http.get<boolean>(`${APP_CONFIG.backendUrls.auth}logout`, { withCredentials: true });
  }

  createAccount(legacyAccount: LegacyAccount): Observable<boolean> {
    // Make sure not to mofidy the object passed in parameter (object reference) when encrypting the password
    const form = Object.assign({}, legacyAccount);
    return this.getPublicKey().pipe(
      tap(
        (publicKey) => {
          form.password = this.encrypt(form.password, publicKey);
        },
      ),
      mergeMap(() => {
        return this._http.post<any>(`${APP_CONFIG.backendUrls.middlewareLegacyAuth}user/`, form);
      }),
      map(
        (res) => {
          return true;
        },
      ),
    );
  }

  validateAccount(token: string) {
    return this._http.post<any>(`${APP_CONFIG.backendUrls.middlewareLegacyAuth}user/validateAccount`, { token });
  }

  getUserInfo(): Observable<IUserInfo> {
    return this._http.get<IUserInfo>(`${APP_CONFIG.backendUrls.auth}user`, { withCredentials: true });
  }

  updateUserInfo(info: IUserInfo): Observable<{ token: string }> {
    return this._http.put<{ token: string }>(
      `${APP_CONFIG.backendUrls.auth}user/update`,
      info,
      { withCredentials: true },
    );
  }

  updateUserPassword(passwordUpdateform: PasswordUpdateForm): Observable<void> {
    // Make sure not to mofidy the object passed in parameter (object reference) when encrypting the password
    const form = Object.assign({}, passwordUpdateform);
    return this.getPublicKey().pipe(
      tap(
        (publicKey) => {
          form.newPassword = this.encrypt(form.newPassword, publicKey);
          form.oldPassword = this.encrypt(form.oldPassword, publicKey);
        },
      ),
      mergeMap(() => {
        return this._http.put<void>(
          `${APP_CONFIG.backendUrls.middlewareLegacyAuth}user/updatePassword`,
          form,
          { withCredentials: true },
        );
      }),
    );
  }

  forgotPassword(email: IPasswordForgottenForm): Observable<void> {
    return this._http.post<void>(`${APP_CONFIG.backendUrls.middlewareLegacyAuth}passwordForgotten`, email).pipe(
      map(
        (res) => {
          return;
        },
      ),
    );
  }

  isPasswordResetTokenValid(token: string): Observable<boolean> {
    // tslint:disable-next-line:max-line-length
    return this._http.get<boolean>(`${APP_CONFIG.backendUrls.middlewareLegacyAuth}isPasswordResetTokenValid?token=${token}`);
  }

  resetPassword(token: string, password: string): Observable<void> {
    // Make sure not to mofidy the object passed in parameter (object reference) when encrypting the password
    let encryptedPassword = '';
    return this.getPublicKey().pipe(
      tap(
        (publicKey) => {
          encryptedPassword = this.encrypt(password, publicKey);
        },
      ),
      mergeMap(() => {
        // tslint:disable-next-line:max-line-length
        return this._http.put<void>(
          `${APP_CONFIG.backendUrls.middlewareLegacyAuth}user/resetPassword`,
          { token, password: encryptedPassword },
          { withCredentials: true },
        );
      }),
    );
  }

  deleteAccount(): Observable<void> {
    return this._http.delete<any>(`${APP_CONFIG.backendUrls.middlewareLegacyAuth}user`, { withCredentials: true }).pipe(
      map(
        (res) => {
          return;
        },
      ),
    );
  }

  getPublicKey(): Observable<any> {
    return this._http.get<any>(`${APP_CONFIG.backendUrls.middlewareLegacyAuth}publicKey`).pipe(
      map(
        (res) => {
          return res.publicKey;
        },
      ),
    );
  }

  /**
   * Takes un unencrypted string as params and returns it encrypted with the public key provided by the backend service
   */
  encrypt(unencrypted: string, publicKey: string): string {
    if (!publicKey) {
      throw new Error('Can\'t encrypt without public key');
    } else {
      const key = new NodeRSA(publicKey);
      const encrypted = key.encrypt(unencrypted, 'base64');
      return encrypted;
    }
  }
}
